# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'rare/ui/components/tabs/games/import_sync/egl_sync_list_group.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_EGLSyncListGroup(object):
    def setupUi(self, EGLSyncListGroup):
        EGLSyncListGroup.setObjectName("EGLSyncListGroup")
        EGLSyncListGroup.resize(461, 206)
        EGLSyncListGroup.setWindowTitle("EGLSyncListGroup")
        EGLSyncListGroup.setTitle("")
        self.egl_sync_list_layout = QtWidgets.QVBoxLayout(EGLSyncListGroup)
        self.egl_sync_list_layout.setObjectName("egl_sync_list_layout")
        self.label = QtWidgets.QLabel(EGLSyncListGroup)
        self.label.setText("")
        self.label.setObjectName("label")
        self.egl_sync_list_layout.addWidget(self.label)
        self.list = QtWidgets.QListWidget(EGLSyncListGroup)
        self.list.setAlternatingRowColors(True)
        self.list.setObjectName("list")
        self.egl_sync_list_layout.addWidget(self.list)
        self.buttons_widget = QtWidgets.QWidget(EGLSyncListGroup)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.buttons_widget.sizePolicy().hasHeightForWidth())
        self.buttons_widget.setSizePolicy(sizePolicy)
        self.buttons_widget.setObjectName("buttons_widget")
        self.buttons_layout = QtWidgets.QHBoxLayout(self.buttons_widget)
        self.buttons_layout.setContentsMargins(0, 0, 0, 0)
        self.buttons_layout.setObjectName("buttons_layout")
        self.select_all_button = QtWidgets.QPushButton(self.buttons_widget)
        self.select_all_button.setObjectName("select_all_button")
        self.buttons_layout.addWidget(self.select_all_button)
        self.select_none_button = QtWidgets.QPushButton(self.buttons_widget)
        self.select_none_button.setObjectName("select_none_button")
        self.buttons_layout.addWidget(self.select_none_button)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.buttons_layout.addItem(spacerItem)
        self.action_button = QtWidgets.QPushButton(self.buttons_widget)
        self.action_button.setText("Action")
        self.action_button.setObjectName("action_button")
        self.buttons_layout.addWidget(self.action_button)
        self.egl_sync_list_layout.addWidget(self.buttons_widget)

        self.retranslateUi(EGLSyncListGroup)

    def retranslateUi(self, EGLSyncListGroup):
        _translate = QtCore.QCoreApplication.translate
        self.list.setSortingEnabled(True)
        self.select_all_button.setText(_translate("EGLSyncListGroup", "Select all"))
        self.select_none_button.setText(_translate("EGLSyncListGroup", "Select none"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    EGLSyncListGroup = QtWidgets.QGroupBox()
    ui = Ui_EGLSyncListGroup()
    ui.setupUi(EGLSyncListGroup)
    EGLSyncListGroup.show()
    sys.exit(app.exec_())
