# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'rare/ui/components/tabs/store/wishlist.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Wishlist(object):
    def setupUi(self, Wishlist):
        Wishlist.setObjectName("Wishlist")
        Wishlist.resize(736, 398)
        Wishlist.setWindowTitle("StackedWidget")
        self.page = QtWidgets.QWidget()
        self.page.setObjectName("page")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.page)
        self.verticalLayout.setObjectName("verticalLayout")
        self.scroll_area = QtWidgets.QScrollArea(self.page)
        self.scroll_area.setWidgetResizable(True)
        self.scroll_area.setObjectName("scroll_area")
        self.scroll_widget = QtWidgets.QWidget()
        self.scroll_widget.setGeometry(QtCore.QRect(0, 0, 716, 378))
        self.scroll_widget.setObjectName("scroll_widget")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.scroll_widget)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.title_label = QtWidgets.QLabel(self.scroll_widget)
        font = QtGui.QFont()
        font.setPointSize(15)
        self.title_label.setFont(font)
        self.title_label.setObjectName("title_label")
        self.verticalLayout_2.addWidget(self.title_label)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.sort_label = QtWidgets.QLabel(self.scroll_widget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sort_label.sizePolicy().hasHeightForWidth())
        self.sort_label.setSizePolicy(sizePolicy)
        self.sort_label.setObjectName("sort_label")
        self.horizontalLayout.addWidget(self.sort_label)
        self.sort_cb = QtWidgets.QComboBox(self.scroll_widget)
        self.sort_cb.setObjectName("sort_cb")
        self.sort_cb.addItem("")
        self.sort_cb.addItem("")
        self.sort_cb.addItem("")
        self.sort_cb.addItem("")
        self.horizontalLayout.addWidget(self.sort_cb)
        self.reverse = QtWidgets.QCheckBox(self.scroll_widget)
        self.reverse.setObjectName("reverse")
        self.horizontalLayout.addWidget(self.reverse)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.filter_label = QtWidgets.QLabel(self.scroll_widget)
        self.filter_label.setObjectName("filter_label")
        self.horizontalLayout.addWidget(self.filter_label)
        self.filter_cb = QtWidgets.QComboBox(self.scroll_widget)
        self.filter_cb.setObjectName("filter_cb")
        self.filter_cb.addItem("")
        self.filter_cb.addItem("")
        self.horizontalLayout.addWidget(self.filter_cb)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.reload_button = QtWidgets.QPushButton(self.scroll_widget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.reload_button.sizePolicy().hasHeightForWidth())
        self.reload_button.setSizePolicy(sizePolicy)
        self.reload_button.setText("")
        self.reload_button.setObjectName("reload_button")
        self.horizontalLayout.addWidget(self.reload_button)
        self.verticalLayout_2.addLayout(self.horizontalLayout)
        self.list_layout = QtWidgets.QVBoxLayout()
        self.list_layout.setObjectName("list_layout")
        self.verticalLayout_2.addLayout(self.list_layout)
        self.no_games_label = QtWidgets.QLabel(self.scroll_widget)
        self.no_games_label.setObjectName("no_games_label")
        self.verticalLayout_2.addWidget(self.no_games_label)
        spacerItem2 = QtWidgets.QSpacerItem(379, 218, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem2)
        self.scroll_area.setWidget(self.scroll_widget)
        self.verticalLayout.addWidget(self.scroll_area)
        Wishlist.addWidget(self.page)

        self.retranslateUi(Wishlist)

    def retranslateUi(self, Wishlist):
        _translate = QtCore.QCoreApplication.translate
        self.title_label.setText(_translate("Wishlist", "Wishlist"))
        self.sort_label.setText(_translate("Wishlist", "Sort by"))
        self.sort_cb.setItemText(0, _translate("Wishlist", "Name"))
        self.sort_cb.setItemText(1, _translate("Wishlist", "Price"))
        self.sort_cb.setItemText(2, _translate("Wishlist", "Developer"))
        self.sort_cb.setItemText(3, _translate("Wishlist", "Discount"))
        self.reverse.setText(_translate("Wishlist", "Reverse"))
        self.filter_label.setText(_translate("Wishlist", "Filter:"))
        self.filter_cb.setItemText(0, _translate("Wishlist", "None"))
        self.filter_cb.setItemText(1, _translate("Wishlist", "Discount"))
        self.no_games_label.setText(_translate("Wishlist", "No games matching your filter"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    Wishlist = QtWidgets.QStackedWidget()
    ui = Ui_Wishlist()
    ui.setupUi(Wishlist)
    Wishlist.show()
    sys.exit(app.exec_())
