from .fetch import FetchWorker
from .install_info import InstallInfoWorker
from .move import MoveWorker
from .uninstall import UninstallWorker
from .verify import VerifyWorker
from .wine_resolver import OriginWineWorker
from .worker import Worker, QueueWorker
