# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'rare/ui/components/tabs/games/game_info/cloud_widget.ui'
#
# Created by: PyQt5 UI code generator 5.15.7
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_CloudWidget(object):
    def setupUi(self, CloudWidget):
        CloudWidget.setObjectName("CloudWidget")
        CloudWidget.resize(251, 93)
        CloudWidget.setWindowTitle("GroupBox")
        self.cloud_layout = QtWidgets.QFormLayout(CloudWidget)
        self.cloud_layout.setObjectName("cloud_layout")
        self.cloud_sync_label = QtWidgets.QLabel(CloudWidget)
        self.cloud_sync_label.setObjectName("cloud_sync_label")
        self.cloud_layout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.cloud_sync_label)
        self.cloud_sync = QtWidgets.QCheckBox(CloudWidget)
        self.cloud_sync.setText("")
        self.cloud_sync.setObjectName("cloud_sync")
        self.cloud_layout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.cloud_sync)

        self.retranslateUi(CloudWidget)

    def retranslateUi(self, CloudWidget):
        _translate = QtCore.QCoreApplication.translate
        CloudWidget.setTitle(_translate("CloudWidget", "Options"))
        self.cloud_sync_label.setText(_translate("CloudWidget", "Sync with cloud"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    CloudWidget = QtWidgets.QGroupBox()
    ui = Ui_CloudWidget()
    ui.setupUi(CloudWidget)
    CloudWidget.show()
    sys.exit(app.exec_())
