# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'rare/ui/components/tabs/downloads/queue_info_widget.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_QueueInfoWidget(object):
    def setupUi(self, QueueInfoWidget):
        QueueInfoWidget.setObjectName("QueueInfoWidget")
        QueueInfoWidget.resize(624, 60)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(QueueInfoWidget.sizePolicy().hasHeightForWidth())
        QueueInfoWidget.setSizePolicy(sizePolicy)
        QueueInfoWidget.setWindowTitle("QueueInfoWidget")
        self.queue_info_layout = QtWidgets.QGridLayout(QueueInfoWidget)
        self.queue_info_layout.setContentsMargins(0, 0, 0, 0)
        self.queue_info_layout.setObjectName("queue_info_layout")
        self.dl_size_label = QtWidgets.QLabel(QueueInfoWidget)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.dl_size_label.setFont(font)
        self.dl_size_label.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.dl_size_label.setObjectName("dl_size_label")
        self.queue_info_layout.addWidget(self.dl_size_label, 1, 3, 1, 1)
        self.title = QtWidgets.QLabel(QueueInfoWidget)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.title.setFont(font)
        self.title.setText("title")
        self.title.setObjectName("title")
        self.queue_info_layout.addWidget(self.title, 0, 1, 1, 4)
        self.local_version_label = QtWidgets.QLabel(QueueInfoWidget)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.local_version_label.setFont(font)
        self.local_version_label.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.local_version_label.setObjectName("local_version_label")
        self.queue_info_layout.addWidget(self.local_version_label, 2, 1, 1, 1)
        self.image_layout = QtWidgets.QVBoxLayout()
        self.image_layout.setSizeConstraint(QtWidgets.QLayout.SetFixedSize)
        self.image_layout.setObjectName("image_layout")
        self.queue_info_layout.addLayout(self.image_layout, 0, 0, 3, 1)
        self.install_size_label = QtWidgets.QLabel(QueueInfoWidget)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.install_size_label.setFont(font)
        self.install_size_label.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.install_size_label.setObjectName("install_size_label")
        self.queue_info_layout.addWidget(self.install_size_label, 2, 3, 1, 1)
        self.install_size = QtWidgets.QLabel(QueueInfoWidget)
        self.install_size.setMinimumSize(QtCore.QSize(160, 0))
        self.install_size.setText("install_size")
        self.install_size.setObjectName("install_size")
        self.queue_info_layout.addWidget(self.install_size, 2, 4, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.queue_info_layout.addItem(spacerItem, 0, 5, 3, 1)
        self.dl_size = QtWidgets.QLabel(QueueInfoWidget)
        self.dl_size.setMinimumSize(QtCore.QSize(160, 0))
        self.dl_size.setText("dl_size")
        self.dl_size.setObjectName("dl_size")
        self.queue_info_layout.addWidget(self.dl_size, 1, 4, 1, 1)
        self.remote_version_label = QtWidgets.QLabel(QueueInfoWidget)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.remote_version_label.setFont(font)
        self.remote_version_label.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.remote_version_label.setObjectName("remote_version_label")
        self.queue_info_layout.addWidget(self.remote_version_label, 1, 1, 1, 1)
        self.local_version = QtWidgets.QLabel(QueueInfoWidget)
        self.local_version.setMinimumSize(QtCore.QSize(160, 0))
        self.local_version.setMaximumSize(QtCore.QSize(160, 16777215))
        self.local_version.setText("local_version")
        self.local_version.setObjectName("local_version")
        self.queue_info_layout.addWidget(self.local_version, 2, 2, 1, 1)
        self.remote_version = QtWidgets.QLabel(QueueInfoWidget)
        self.remote_version.setMinimumSize(QtCore.QSize(160, 0))
        self.remote_version.setMaximumSize(QtCore.QSize(160, 16777215))
        self.remote_version.setText("remote_version")
        self.remote_version.setObjectName("remote_version")
        self.queue_info_layout.addWidget(self.remote_version, 1, 2, 1, 1)

        self.retranslateUi(QueueInfoWidget)

    def retranslateUi(self, QueueInfoWidget):
        _translate = QtCore.QCoreApplication.translate
        self.dl_size_label.setText(_translate("QueueInfoWidget", "Download size"))
        self.local_version_label.setText(_translate("QueueInfoWidget", "Local version"))
        self.install_size_label.setText(_translate("QueueInfoWidget", "Installed size"))
        self.remote_version_label.setText(_translate("QueueInfoWidget", "Remote version"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    QueueInfoWidget = QtWidgets.QWidget()
    ui = Ui_QueueInfoWidget()
    ui.setupUi(QueueInfoWidget)
    QueueInfoWidget.show()
    sys.exit(app.exec_())
