"""
Module that overloads and monkeypatches legendary's classes/methods to work with Rare

Files with the 'api_' prefix are not part of legendary's source, and contain facilities relating to Rare.
"""