# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'rare/ui/components/dialogs/install_dialog_advanced.ui'
#
# Created by: PyQt5 UI code generator 5.15.7
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_InstallDialogAdvanced(object):
    def setupUi(self, InstallDialogAdvanced):
        InstallDialogAdvanced.setObjectName("InstallDialogAdvanced")
        InstallDialogAdvanced.resize(379, 208)
        InstallDialogAdvanced.setWindowTitle("InstallDialogAdvanced")
        self.install_dialog_advanced_layout = QtWidgets.QFormLayout(InstallDialogAdvanced)
        self.install_dialog_advanced_layout.setObjectName("install_dialog_advanced_layout")
        self.max_workers_label = QtWidgets.QLabel(InstallDialogAdvanced)
        self.max_workers_label.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.max_workers_label.setObjectName("max_workers_label")
        self.install_dialog_advanced_layout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.max_workers_label)
        self.max_workers_layout = QtWidgets.QHBoxLayout()
        self.max_workers_layout.setObjectName("max_workers_layout")
        self.max_workers_spin = QtWidgets.QSpinBox(InstallDialogAdvanced)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.max_workers_spin.sizePolicy().hasHeightForWidth())
        self.max_workers_spin.setSizePolicy(sizePolicy)
        self.max_workers_spin.setObjectName("max_workers_spin")
        self.max_workers_layout.addWidget(self.max_workers_spin)
        self.max_workers_info = QtWidgets.QLabel(InstallDialogAdvanced)
        font = QtGui.QFont()
        font.setItalic(True)
        self.max_workers_info.setFont(font)
        self.max_workers_info.setObjectName("max_workers_info")
        self.max_workers_layout.addWidget(self.max_workers_info)
        self.install_dialog_advanced_layout.setLayout(0, QtWidgets.QFormLayout.FieldRole, self.max_workers_layout)
        self.max_memory_label = QtWidgets.QLabel(InstallDialogAdvanced)
        self.max_memory_label.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.max_memory_label.setObjectName("max_memory_label")
        self.install_dialog_advanced_layout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.max_memory_label)
        self.max_memory_layout = QtWidgets.QHBoxLayout()
        self.max_memory_layout.setObjectName("max_memory_layout")
        self.max_memory_spin = QtWidgets.QSpinBox(InstallDialogAdvanced)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.max_memory_spin.sizePolicy().hasHeightForWidth())
        self.max_memory_spin.setSizePolicy(sizePolicy)
        self.max_memory_spin.setMinimum(0)
        self.max_memory_spin.setMaximum(10240)
        self.max_memory_spin.setSingleStep(128)
        self.max_memory_spin.setProperty("value", 1024)
        self.max_memory_spin.setObjectName("max_memory_spin")
        self.max_memory_layout.addWidget(self.max_memory_spin)
        self.max_memory_info = QtWidgets.QLabel(InstallDialogAdvanced)
        font = QtGui.QFont()
        font.setItalic(True)
        self.max_memory_info.setFont(font)
        self.max_memory_info.setObjectName("max_memory_info")
        self.max_memory_layout.addWidget(self.max_memory_info)
        self.install_dialog_advanced_layout.setLayout(1, QtWidgets.QFormLayout.FieldRole, self.max_memory_layout)
        self.install_prereqs_label = QtWidgets.QLabel(InstallDialogAdvanced)
        self.install_prereqs_label.setObjectName("install_prereqs_label")
        self.install_dialog_advanced_layout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.install_prereqs_label)
        self.install_prereqs_check = QtWidgets.QCheckBox(InstallDialogAdvanced)
        font = QtGui.QFont()
        font.setItalic(True)
        self.install_prereqs_check.setFont(font)
        self.install_prereqs_check.setText("")
        self.install_prereqs_check.setChecked(False)
        self.install_prereqs_check.setObjectName("install_prereqs_check")
        self.install_dialog_advanced_layout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.install_prereqs_check)
        self.dl_optimizations_label = QtWidgets.QLabel(InstallDialogAdvanced)
        self.dl_optimizations_label.setObjectName("dl_optimizations_label")
        self.install_dialog_advanced_layout.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.dl_optimizations_label)
        self.dl_optimizations_check = QtWidgets.QCheckBox(InstallDialogAdvanced)
        self.dl_optimizations_check.setText("")
        self.dl_optimizations_check.setChecked(False)
        self.dl_optimizations_check.setObjectName("dl_optimizations_check")
        self.install_dialog_advanced_layout.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.dl_optimizations_check)
        self.force_download_label = QtWidgets.QLabel(InstallDialogAdvanced)
        self.force_download_label.setObjectName("force_download_label")
        self.install_dialog_advanced_layout.setWidget(4, QtWidgets.QFormLayout.LabelRole, self.force_download_label)
        self.force_download_check = QtWidgets.QCheckBox(InstallDialogAdvanced)
        self.force_download_check.setText("")
        self.force_download_check.setObjectName("force_download_check")
        self.install_dialog_advanced_layout.setWidget(4, QtWidgets.QFormLayout.FieldRole, self.force_download_check)
        self.ignore_space_label = QtWidgets.QLabel(InstallDialogAdvanced)
        self.ignore_space_label.setObjectName("ignore_space_label")
        self.install_dialog_advanced_layout.setWidget(5, QtWidgets.QFormLayout.LabelRole, self.ignore_space_label)
        self.ignore_space_check = QtWidgets.QCheckBox(InstallDialogAdvanced)
        font = QtGui.QFont()
        font.setItalic(True)
        self.ignore_space_check.setFont(font)
        self.ignore_space_check.setObjectName("ignore_space_check")
        self.install_dialog_advanced_layout.setWidget(5, QtWidgets.QFormLayout.FieldRole, self.ignore_space_check)
        self.download_only_label = QtWidgets.QLabel(InstallDialogAdvanced)
        self.download_only_label.setObjectName("download_only_label")
        self.install_dialog_advanced_layout.setWidget(6, QtWidgets.QFormLayout.LabelRole, self.download_only_label)
        self.download_only_check = QtWidgets.QCheckBox(InstallDialogAdvanced)
        font = QtGui.QFont()
        font.setItalic(True)
        self.download_only_check.setFont(font)
        self.download_only_check.setObjectName("download_only_check")
        self.install_dialog_advanced_layout.setWidget(6, QtWidgets.QFormLayout.FieldRole, self.download_only_check)

        self.retranslateUi(InstallDialogAdvanced)

    def retranslateUi(self, InstallDialogAdvanced):
        _translate = QtCore.QCoreApplication.translate
        self.max_workers_label.setText(_translate("InstallDialogAdvanced", "Max workers"))
        self.max_workers_info.setText(_translate("InstallDialogAdvanced", "Less is slower. (0: Default)"))
        self.max_memory_label.setText(_translate("InstallDialogAdvanced", "Max shared memory"))
        self.max_memory_spin.setSuffix(_translate("InstallDialogAdvanced", "MiB"))
        self.max_memory_info.setText(_translate("InstallDialogAdvanced", "Less is slower (0: Default)"))
        self.install_prereqs_label.setText(_translate("InstallDialogAdvanced", "Install prerequisites"))
        self.dl_optimizations_label.setText(_translate("InstallDialogAdvanced", "Enable reordering"))
        self.force_download_label.setText(_translate("InstallDialogAdvanced", "Force redownload"))
        self.ignore_space_label.setText(_translate("InstallDialogAdvanced", "Ignore free space"))
        self.ignore_space_check.setText(_translate("InstallDialogAdvanced", "Use with caution!"))
        self.download_only_label.setText(_translate("InstallDialogAdvanced", "Download only"))
        self.download_only_check.setText(_translate("InstallDialogAdvanced", "Do not try to install."))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    InstallDialogAdvanced = QtWidgets.QWidget()
    ui = Ui_InstallDialogAdvanced()
    ui.setupUi(InstallDialogAdvanced)
    InstallDialogAdvanced.show()
    sys.exit(app.exec_())
